@@ -112,6 +112,8 @@ class Main(object):
                                   'port': 8022,
                                   'memcache': None,
                                   'max-children': "20",
+                                  'auth-timeout': "60",
+                                  'negotiation-timeout': "0",
                                   'log-file': None,
                                   'syslog': 'no',
                                   'verbose': 'no',
@@ -281,6 +283,22 @@ class Main(object):
         except ValueError:
             parser.error('max-children: invalid value, integer expected')
 
+        try:
+            options.auth_timeout = int(config.get('sftpcloudfs', 'auth-timeout'))
+        except ValueError:
+            parser.error('auth-timeout: invalid value, integer expected')
+
+        if options.auth_timeout <= 0:
+            parser.error('auth-timeout: invalid value')
+
+        try:
+            options.negotiation_timeout = int(config.get('sftpcloudfs', 'negotiation-timeout'))
+        except ValueError:
+            parser.error('negotiation-timeout: invalid value, integer expected')
+
+        if options.negotiation_timeout < 0:
+            parser.error('negotiation-timeout: invalid value')
+
         try:
             options.split_size = int(config.get('sftpcloudfs', 'split-large-files'))*10**6
         except ValueError:
@@ -336,6 +354,8 @@ class Main(object):
                                           no_scp=self.options.no_scp,
                                           split_size=self.options.split_size,
                                           hide_part_dir=self.options.hide_part_dir,
+                                          auth_timeout=self.options.auth_timeout,
+                                          negotiation_timeout=self.options.negotiation_timeout,
                                           )
 
         dc = daemon.DaemonContext()
